/** 
 * FractalNoteGenerator.java
 *
 * Written by John Waymouth as part of the Fractals and Art IQP by John 
 * Waymouth and Steve Conte.
 *
 * Fractals and Art: Music and Fractals
 *
 * 
 * This class is like the FractalMultipleNoteGenerator, except that it generates 
 * 4-voice chords that are all in the key of C Major.  This means that the chords 
 * will not sound as dissonant, and that the brain may try to follow individual 
 * voices. 
 *
 */

import java.util.*;
import javax.sound.midi.*;

public class FractalChordGenerator {
    private int key;
    private long duration;
    private IteratedFunction func;

    /**
     * @param key the first note in the desired key
     * @param duration the duration of each note, in milliseconds
     * @param func the source of fractal data
     */
    public FractalChordGenerator(int key, long duration,
				IteratedFunction func) {
	this.key=key;
	this.duration=duration;
	this.func=func;
    }

    public static void main(String[] args) {
	IteratedFunction func=new RandomFunction(System.currentTimeMillis());
	FractalChordGenerator fng=new FractalChordGenerator(48,500,func);

	fng.generate(20000);
	System.exit(0);
    }

    /**
     * Generate notes using the fractal number source in func.  Notes are played
     * using the computer's speakers and MIDI system.
     *
     * @param length how long to play notes for, in milliseconds
     */
    public void generate(long length) {
	/* Create lists of chords.  Each chord is specified by listing the offset
	 * above the bass note, in half-steps.  These chords are chosen from
	 * various possible positionings of the four voices.  Note that the last 
	 * minor chord is in first inversion for added fun. */ 
	int majorChords[][]=new int[][]{{0,16,19,24},{0,7,16,19},{0,12,19,28}};
	int minorChords[][]=new int[][]{{0,15,19,24},{0,7,19,24},{0,12,19,27},{0,4,12,21}};

	/* This array tells whether a chord starting at this step should be
	 * major or minor (true means major) */
	boolean major[]=new boolean[]{true,false,false,true,true,false,false,true};

	/* This array gives the offsets in half-steps of each note of a major 
	 * scale */
	int steps[]=new int[]{0,2,4,5,7,9,11,12};

	try {

	    long endTime=System.currentTimeMillis() + length;
	    long startTime=System.currentTimeMillis();
	    Synthesizer synth=MidiSystem.getSynthesizer();
	    synth.open();
	    Receiver synthReceiver=synth.getReceiver();
	    
	    ShortMessage cmd=new ShortMessage();
	    while(System.currentTimeMillis() < endTime) {
		/* use the chaos function to generate the bass note, by choosing
		 * a step of the scale, finding its half-step offset from steps[], 
		 * and using that to calculate the MIDI note value */
		int step=func.nextValue()%8;
		int bassNote=key + steps[step];
		int []chord;

		/* choose the chord.  Make the decision by using more data from
		 * the chaos function */
		if (major[step])
		    chord=majorChords[func.nextValue() % majorChords.length];
		else
		    chord=minorChords[func.nextValue() % minorChords.length];
		    
		/* turn on all notes of the chord */
		for (int i=0;i<chord.length;i++) {
		    cmd.setMessage(cmd.NOTE_ON,bassNote+chord[i],64);
		    synthReceiver.send(cmd,-1);
		}
		
		/* wait to send the note off */
		Thread.sleep(Math.min(endTime-System.currentTimeMillis(),duration));
		
		/* turn off all notes of the chord */
		for (int i=0;i<chord.length;i++) {
		    cmd.setMessage(cmd.NOTE_OFF,bassNote+chord[i],64);
		    synthReceiver.send(cmd,-1);
		}
	    }

	    synth.close();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
