/** 
 * FractalMultipleNoteGenerator.java
 *
 * Written by John Waymouth as part of the Fractals and Art IQP by John 
 * Waymouth and Steve Conte.
 *
 * Fractals and Art: Music and Fractals
 *
 * This class is like the FractalNoteGenerator except that it plays 3 notes 
 * at once, generating a "chord".  This demonstrates that fractals tend not to 
 * be good for generating nice-sounding harmonies.  
 *
 */

import java.util.*;
import javax.sound.midi.*;

public class FractalMultipleNoteGenerator {
    private int minNote;
    private int spread;
    private long duration;
    private IteratedFunction func;

    /**
     * @param minNote the lowest MIDI note value to generate
     * @param maxNote the highest MIDI note value to generate
     * @param duration the duration of each note, in milliseconds
     * @param func the source of fractal data
     */
    public FractalMultipleNoteGenerator(int minNote, int maxNote, long duration,
				IteratedFunction func) {
	this.minNote=minNote;
	this.spread=maxNote-minNote;
	this.duration=duration;
	this.func=func;
    }

    public static void main(String[] args) {
	IteratedFunction func=new RandomFunction(System.currentTimeMillis());
	/* generate all notes in the octave starting at middle C */
	FractalMultipleNoteGenerator fmng=
	    new FractalMultipleNoteGenerator(60,72,500,func);

	fmng.generate(20000);
	System.exit(0);
    }

    /**
     * Generate notes using the fractal number source in func.  Notes are played
     * using the computer's speakers and MIDI system.
     *
     * @param length how long to play notes for, in milliseconds
     */
    public void generate(long length) {
	try {

	    long endTime=System.currentTimeMillis() + length;
	    long startTime=System.currentTimeMillis();
	    Synthesizer synth=MidiSystem.getSynthesizer();
	    synth.open();
	    Receiver synthReceiver=synth.getReceiver();
	    
	    ShortMessage cmd=new ShortMessage();
	    int noteValues[]=new int[3];

	    while(System.currentTimeMillis() < endTime) {
		/* use the chaos function to generate the next note values, 
		 * which are numbers between minNote and minNote+spread */
		for (int i=0;i<3;i++)
		    noteValues[i]=func.nextValue()%(spread+1) + minNote;
		
		/* turn all notes on */
		for (int i=0;i<3;i++) {
		    cmd.setMessage(cmd.NOTE_ON,noteValues[i],64);
		    synthReceiver.send(cmd,-1);
		}

		/* wait to send the note off */
		Thread.sleep(Math.min(endTime-System.currentTimeMillis(),duration));
		
		/* turn the note off */
		for (int i=0;i<3;i++) {
		    cmd.setMessage(cmd.NOTE_OFF,noteValues[i],64);
		    synthReceiver.send(cmd,-1);
		}
	    }

	    synth.close();
	} catch (Exception e) {
	    System.out.println("Exception!");
	    e.printStackTrace();
	    System.exit(-1);
	}
    }
}
