/** 
 * FractalNoteGenerator.java
 *
 * Written by John Waymouth as part of the Fractals and Art IQP by John 
 * Waymouth and Steve Conte.
 *
 * Fractals and Art: Music and Fractals
 *
 * This java class is an example of a program that generates MIDI notes using a
 * chaos function.  Chaos functions are implemented through the IteratedFunction
 * interface, so that they can be interchanged.  
 *
 */

import java.util.*;
import javax.sound.midi.*;

public class FractalNoteGenerator {
    private int minNote;
    private int spread;
    private long duration;
    private IteratedFunction func;

    /**
     * @param minNote the lowest MIDI note value to generate
     * @param maxNote the highest MIDI note value to generate
     * @param duration the duration of each note, in milliseconds
     * @param func the source of fractal data
     */
    public FractalNoteGenerator(int minNote, int maxNote, long duration,
				IteratedFunction func) {
	this.minNote=minNote;
	this.spread=maxNote-minNote;
	this.duration=duration;
	this.func=func;
    }

    public static void main(String[] args) {
	IteratedFunction func=new RandomFunction(System.currentTimeMillis());
	FractalNoteGenerator fng=new FractalNoteGenerator(48,72,500,func);

	fng.generate(20000);
	System.exit(0);
    }

    /**
     * Generate notes using the fractal number source in func.  Notes are played
     * using the computer's speakers and MIDI system.
     *
     * @param length how long to play notes for, in milliseconds
     */
    public void generate(long length) {
	try {

	    long endTime=System.currentTimeMillis() + length;
	    long startTime=System.currentTimeMillis();
	    Synthesizer synth=MidiSystem.getSynthesizer();
	    synth.open();
	    Receiver synthReceiver=synth.getReceiver();
	    
	    ShortMessage cmd=new ShortMessage();
	    while(System.currentTimeMillis() < endTime) {
		/* use the chaos function to generate the next note value, which is
		 * a number between minNote and minNote+spread */
		int noteValue=func.nextValue()%(spread+1) + minNote;
		
		/* turn this note on */
		cmd.setMessage(cmd.NOTE_ON,noteValue,64);
		synthReceiver.send(cmd,-1);
		
		/* wait to send the note off */
		Thread.sleep(Math.min(endTime-System.currentTimeMillis(),duration));
		
		/* turn the note off */
		cmd.setMessage(cmd.NOTE_OFF,noteValue,64);
		synthReceiver.send(cmd,-1);
	    }

	    synth.close();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
